# -*- coding: utf-8 -*-

"""
    mundiapi.models.error_exception

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from ..api_helper import APIHelper
import mundiapi.exceptions.api_exception

class ErrorException(mundiapi.exceptions.api_exception.APIException):
    def __init__(self, reason, context):
        """Constructor for the ErrorException class

        Args:
            reason (string): The reason (or error message) for the Exception
                to be raised.
            context (HttpContext):  The HttpContext of the API call.

        """
        super(ErrorException, self).__init__(reason, context)
        dictionary = APIHelper.json_deserialize(self.context.response.raw_body)
        if isinstance(dictionary, dict):
            self.unbox(dictionary)

    def unbox(self, dictionary):
        """Populates the properties of this object by extracting them from a dictionary.

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        """
        self.message = dictionary.get("message")
        self.errors = dictionary.get("errors")
        self.request = dictionary.get("request")



# -*- coding: utf-8 -*-

"""
    mundiapi.models.paging_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class PagingResponse(object):

    """Implementation of the 'PagingResponse' model.

    Object used for returning lists of objects with pagination

    Attributes:
        total (int): Total number of pages
        previous (string): Previous page
        next (string): Next page

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "total" : "total",
        "previous" : "previous",
        "next" : "next"
    }

    def __init__(self,
                 total=None,
                 previous=None,
                 next=None):
        """Constructor for the PagingResponse class"""

        # Initialize members of the class
        self.total = total
        self.previous = previous
        self.next = next


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        total = dictionary.get("total")
        previous = dictionary.get("previous")
        next = dictionary.get("next")

        # Return an object of this model
        return cls(total,
                   previous,
                   next)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_checkout_card_installment_option_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateCheckoutCardInstallmentOptionRequest(object):

    """Implementation of the 'CreateCheckoutCardInstallmentOptionRequest' model.

    Options for card installment

    Attributes:
        number (int): Installment quantity
        total (int): Total amount

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "number" : "number",
        "total" : "total"
    }

    def __init__(self,
                 number=None,
                 total=None):
        """Constructor for the CreateCheckoutCardInstallmentOptionRequest class"""

        # Initialize members of the class
        self.number = number
        self.total = total


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        number = dictionary.get("number")
        total = dictionary.get("total")

        # Return an object of this model
        return cls(number,
                   total)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_address_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_customer_response

class GetAddressResponse(object):

    """Implementation of the 'GetAddressResponse' model.

    Response object for getting an Address

    Attributes:
        id (string): TODO: type description here.
        street (string): TODO: type description here.
        number (string): TODO: type description here.
        complement (string): TODO: type description here.
        zip_code (string): TODO: type description here.
        neighborhood (string): TODO: type description here.
        city (string): TODO: type description here.
        state (string): TODO: type description here.
        country (string): TODO: type description here.
        status (string): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        customer (GetCustomerResponse): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        deleted_at (datetime): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "street" : "street",
        "number" : "number",
        "complement" : "complement",
        "zip_code" : "zip_code",
        "neighborhood" : "neighborhood",
        "city" : "city",
        "state" : "state",
        "country" : "country",
        "status" : "status",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "customer" : "customer",
        "metadata" : "metadata",
        "deleted_at" : "deleted_at"
    }

    def __init__(self,
                 id=None,
                 street=None,
                 number=None,
                 complement=None,
                 zip_code=None,
                 neighborhood=None,
                 city=None,
                 state=None,
                 country=None,
                 status=None,
                 created_at=None,
                 updated_at=None,
                 customer=None,
                 metadata=None,
                 deleted_at=None):
        """Constructor for the GetAddressResponse class"""

        # Initialize members of the class
        self.id = id
        self.street = street
        self.number = number
        self.complement = complement
        self.zip_code = zip_code
        self.neighborhood = neighborhood
        self.city = city
        self.state = state
        self.country = country
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.customer = customer
        self.metadata = metadata
        self.deleted_at = APIHelper.RFC3339DateTime(deleted_at) if deleted_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        street = dictionary.get("street")
        number = dictionary.get("number")
        complement = dictionary.get("complement")
        zip_code = dictionary.get("zip_code")
        neighborhood = dictionary.get("neighborhood")
        city = dictionary.get("city")
        state = dictionary.get("state")
        country = dictionary.get("country")
        status = dictionary.get("status")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        customer = mundiapi.models.get_customer_response.GetCustomerResponse.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        metadata = dictionary.get("metadata")
        deleted_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("deleted_at")).datetime if dictionary.get("deleted_at") else None

        # Return an object of this model
        return cls(id,
                   street,
                   number,
                   complement,
                   zip_code,
                   neighborhood,
                   city,
                   state,
                   country,
                   status,
                   created_at,
                   updated_at,
                   customer,
                   metadata,
                   deleted_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_setup_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class GetSetupResponse(object):

    """Implementation of the 'GetSetupResponse' model.

    Response object for getting the setup from a subscription

    Attributes:
        id (string): TODO: type description here.
        description (string): TODO: type description here.
        amount (int): TODO: type description here.
        status (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "description" : "description",
        "amount" : "amount",
        "status" : "status"
    }

    def __init__(self,
                 id=None,
                 description=None,
                 amount=None,
                 status=None):
        """Constructor for the GetSetupResponse class"""

        # Initialize members of the class
        self.id = id
        self.description = description
        self.amount = amount
        self.status = status


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        description = dictionary.get("description")
        amount = dictionary.get("amount")
        status = dictionary.get("status")

        # Return an object of this model
        return cls(id,
                   description,
                   amount,
                   status)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.update_subscription_payment_method_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_card_request

class UpdateSubscriptionPaymentMethodRequest(object):

    """Implementation of the 'UpdateSubscriptionPaymentMethodRequest' model.

    Request for updating a subscription's payment method

    Attributes:
        payment_method (string): The new payment method
        card_id (string): Card id
        card (CreateCardRequest): Card data

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payment_method" : "payment_method",
        "card_id" : "card_id",
        "card" : "card"
    }

    def __init__(self,
                 payment_method=None,
                 card_id=None,
                 card=None):
        """Constructor for the UpdateSubscriptionPaymentMethodRequest class"""

        # Initialize members of the class
        self.payment_method = payment_method
        self.card_id = card_id
        self.card = card


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        payment_method = dictionary.get("payment_method")
        card_id = dictionary.get("card_id")
        card = mundiapi.models.create_card_request.CreateCardRequest.from_dictionary(dictionary.get("card")) if dictionary.get("card") else None

        # Return an object of this model
        return cls(payment_method,
                   card_id,
                   card)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_order_item_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.get_seller_response

class GetOrderItemResponse(object):

    """Implementation of the 'GetOrderItemResponse' model.

    Response object for getting an order item

    Attributes:
        amount (int): TODO: type description here.
        description (string): TODO: type description here.
        quantity (int): TODO: type description here.
        get_seller_response (GetSellerResponse): Seller data

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount" : "amount",
        "description" : "description",
        "quantity" : "quantity",
        "get_seller_response" : "GetSellerResponse"
    }

    def __init__(self,
                 amount=None,
                 description=None,
                 quantity=None,
                 get_seller_response=None):
        """Constructor for the GetOrderItemResponse class"""

        # Initialize members of the class
        self.amount = amount
        self.description = description
        self.quantity = quantity
        self.get_seller_response = get_seller_response


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = dictionary.get("amount")
        description = dictionary.get("description")
        quantity = dictionary.get("quantity")
        get_seller_response = mundiapi.models.get_seller_response.GetSellerResponse.from_dictionary(dictionary.get("GetSellerResponse")) if dictionary.get("GetSellerResponse") else None

        # Return an object of this model
        return cls(amount,
                   description,
                   quantity,
                   get_seller_response)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_order_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_order_item_request
import mundiapi.models.create_customer_request
import mundiapi.models.create_payment_request
import mundiapi.models.create_shipping_request

class CreateOrderRequest(object):

    """Implementation of the 'CreateOrderRequest' model.

    Request for creating an order

    Attributes:
        items (list of CreateOrderItemRequest): Items
        customer (CreateCustomerRequest): Customer
        payments (list of CreatePaymentRequest): Payment data
        code (string): The order code
        customer_id (string): The customer id
        shipping (CreateShippingRequest): Shipping data
        metadata (dict<object, string>): Metadata
        antifraud_enabled (bool): Defines whether the order will go through
            anti-fraud

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "items" : "items",
        "customer" : "customer",
        "payments" : "payments",
        "code" : "code",
        "customer_id" : "customer_id",
        "shipping" : "shipping",
        "metadata" : "metadata",
        "antifraud_enabled" : "antifraud_enabled"
    }

    def __init__(self,
                 items=None,
                 customer=None,
                 payments=None,
                 code=None,
                 customer_id=None,
                 shipping=None,
                 metadata=None,
                 antifraud_enabled=None):
        """Constructor for the CreateOrderRequest class"""

        # Initialize members of the class
        self.items = items
        self.customer = customer
        self.payments = payments
        self.code = code
        self.customer_id = customer_id
        self.shipping = shipping
        self.metadata = metadata
        self.antifraud_enabled = antifraud_enabled


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        items = None
        if dictionary.get("items") != None:
            items = list()
            for structure in dictionary.get("items"):
                items.append(mundiapi.models.create_order_item_request.CreateOrderItemRequest.from_dictionary(structure))
        customer = mundiapi.models.create_customer_request.CreateCustomerRequest.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        payments = None
        if dictionary.get("payments") != None:
            payments = list()
            for structure in dictionary.get("payments"):
                payments.append(mundiapi.models.create_payment_request.CreatePaymentRequest.from_dictionary(structure))
        code = dictionary.get("code")
        customer_id = dictionary.get("customer_id")
        shipping = mundiapi.models.create_shipping_request.CreateShippingRequest.from_dictionary(dictionary.get("shipping")) if dictionary.get("shipping") else None
        metadata = dictionary.get("metadata")
        antifraud_enabled = dictionary.get("antifraud_enabled")

        # Return an object of this model
        return cls(items,
                   customer,
                   payments,
                   code,
                   customer_id,
                   shipping,
                   metadata,
                   antifraud_enabled)





# -*- coding: utf-8 -*-

"""
    mundiapi.controllers.orders_controller

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.get_order_response import GetOrderResponse
from ..models.list_order_response import ListOrderResponse

class OrdersController(BaseController):

    """A Controller to access Endpoints in the mundiapi API."""


    def get_order(self,
                  order_id):
        """Does a GET request to /orders/{order_id}.

        Gets an order

        Args:
            order_id (string): Order id

        Returns:
            GetOrderResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/orders/{order_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'order_id': order_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetOrderResponse.from_dictionary)

    def create_order(self,
                     body):
        """Does a POST request to /orders.

        Creates a new Order

        Args:
            body (CreateOrderRequest): Request for creating an order

        Returns:
            GetOrderResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/orders'
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetOrderResponse.from_dictionary)

    def update_order_metadata(self,
                              order_id,
                              request):
        """Does a PATCH request to /Orders/{order_id}/metadata.

        Updates the metadata from an order

        Args:
            order_id (string): The order id
            request (UpdateMetadataRequest): Request for updating the order
                metadata

        Returns:
            GetOrderResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/Orders/{order_id}/metadata'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'order_id': order_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetOrderResponse.from_dictionary)

    def get_orders(self,
                   page=None,
                   size=None,
                   code=None,
                   status=None,
                   created_since=None,
                   created_until=None,
                   customer_id=None):
        """Does a GET request to /orders.

        Gets all orders

        Args:
            page (int, optional): Page number
            size (int, optional): Page size
            code (string, optional): Filter for order's code
            status (string, optional): Filter for order's status
            created_since (datetime, optional): Filter for order's creation
                date start range
            created_until (datetime, optional): Filter for order's creation
                date end range
            customer_id (string, optional): Filter for order's customer id

        Returns:
            ListOrderResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/orders'
        _query_parameters = {
            'page': page,
            'size': size,
            'code': code,
            'status': status,
            'created_since': APIHelper.RFC3339DateTime(created_since),
            'created_until': APIHelper.RFC3339DateTime(created_until),
            'customer_id': customer_id
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ListOrderResponse.from_dictionary)



# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_billing_address_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class GetBillingAddressResponse(object):

    """Implementation of the 'GetBillingAddressResponse' model.

    Response object for getting a billing address

    Attributes:
        street (string): TODO: type description here.
        number (string): TODO: type description here.
        zip_code (string): TODO: type description here.
        neighborhood (string): TODO: type description here.
        city (string): TODO: type description here.
        state (string): TODO: type description here.
        country (string): TODO: type description here.
        complement (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "street" : "street",
        "number" : "number",
        "zip_code" : "zip_code",
        "neighborhood" : "neighborhood",
        "city" : "city",
        "state" : "state",
        "country" : "country",
        "complement" : "complement"
    }

    def __init__(self,
                 street=None,
                 number=None,
                 zip_code=None,
                 neighborhood=None,
                 city=None,
                 state=None,
                 country=None,
                 complement=None):
        """Constructor for the GetBillingAddressResponse class"""

        # Initialize members of the class
        self.street = street
        self.number = number
        self.zip_code = zip_code
        self.neighborhood = neighborhood
        self.city = city
        self.state = state
        self.country = country
        self.complement = complement


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        street = dictionary.get("street")
        number = dictionary.get("number")
        zip_code = dictionary.get("zip_code")
        neighborhood = dictionary.get("neighborhood")
        city = dictionary.get("city")
        state = dictionary.get("state")
        country = dictionary.get("country")
        complement = dictionary.get("complement")

        # Return an object of this model
        return cls(street,
                   number,
                   zip_code,
                   neighborhood,
                   city,
                   state,
                   country,
                   complement)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_seller_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.get_address_response

class GetSellerResponse(object):

    """Implementation of the 'GetSellerResponse' model.

    TODO: type model description here.

    Attributes:
        id (string): Identification
        name (string): TODO: type description here.
        code (string): TODO: type description here.
        document (string): TODO: type description here.
        description (string): Description
        status (string): Status
        created_at (string): Creation date
        updated_at (string): Updated date
        address (GetAddressResponse): Address
        metadata (object): Metadata
        deleted_at (string): Deleted date

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "name" : "name",
        "code" : "code",
        "document" : "document",
        "description" : "description",
        "status" : "Status",
        "created_at" : "CreatedAt",
        "updated_at" : "UpdatedAt",
        "address" : "Address",
        "metadata" : "Metadata",
        "deleted_at" : "DeletedAt"
    }

    def __init__(self,
                 id=None,
                 name=None,
                 code=None,
                 document=None,
                 description=None,
                 status=None,
                 created_at=None,
                 updated_at=None,
                 address=None,
                 metadata=None,
                 deleted_at=None):
        """Constructor for the GetSellerResponse class"""

        # Initialize members of the class
        self.id = id
        self.name = name
        self.code = code
        self.document = document
        self.description = description
        self.status = status
        self.created_at = created_at
        self.updated_at = updated_at
        self.address = address
        self.metadata = metadata
        self.deleted_at = deleted_at


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        name = dictionary.get("name")
        code = dictionary.get("code")
        document = dictionary.get("document")
        description = dictionary.get("description")
        status = dictionary.get("Status")
        created_at = dictionary.get("CreatedAt")
        updated_at = dictionary.get("UpdatedAt")
        address = mundiapi.models.get_address_response.GetAddressResponse.from_dictionary(dictionary.get("Address")) if dictionary.get("Address") else None
        metadata = dictionary.get("Metadata")
        deleted_at = dictionary.get("DeletedAt")

        # Return an object of this model
        return cls(id,
                   name,
                   code,
                   document,
                   description,
                   status,
                   created_at,
                   updated_at,
                   address,
                   metadata,
                   deleted_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_credit_card_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_card_request

class CreateCreditCardPaymentRequest(object):

    """Implementation of the 'CreateCreditCardPaymentRequest' model.

    The settings for creating a credit card payment

    Attributes:
        installments (int): Number of installments
        statement_descriptor (string): The text that will be shown on the
            credit card's statement
        card (CreateCardRequest): Credit card data
        retries (int): Number of retries
        update_subscription_card (bool): Indicates if the card from the
            subscription must be updated
        card_id (string): The credit card id
        card_token (string): TODO: type description here.
        recurrence (bool): Indicates a recurrence
        capture (bool): Indicates if the operation should be only
            authorization or auth and capture.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "installments" : "installments",
        "statement_descriptor" : "statement_descriptor",
        "card" : "card",
        "retries" : "retries",
        "update_subscription_card" : "update_subscription_card",
        "card_id" : "card_id",
        "card_token" : "card_token",
        "recurrence" : "recurrence",
        "capture" : "capture"
    }

    def __init__(self,
                 installments=1,
                 statement_descriptor=None,
                 card=None,
                 retries=None,
                 update_subscription_card=None,
                 card_id=None,
                 card_token=None,
                 recurrence=None,
                 capture=True):
        """Constructor for the CreateCreditCardPaymentRequest class"""

        # Initialize members of the class
        self.installments = installments
        self.statement_descriptor = statement_descriptor
        self.card = card
        self.retries = retries
        self.update_subscription_card = update_subscription_card
        self.card_id = card_id
        self.card_token = card_token
        self.recurrence = recurrence
        self.capture = capture


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        installments = dictionary.get("installments") if dictionary.get("installments") else 1
        statement_descriptor = dictionary.get("statement_descriptor")
        card = mundiapi.models.create_card_request.CreateCardRequest.from_dictionary(dictionary.get("card")) if dictionary.get("card") else None
        retries = dictionary.get("retries")
        update_subscription_card = dictionary.get("update_subscription_card")
        card_id = dictionary.get("card_id")
        card_token = dictionary.get("card_token")
        recurrence = dictionary.get("recurrence")
        capture = dictionary.get("capture") if dictionary.get("capture") else True

        # Return an object of this model
        return cls(installments,
                   statement_descriptor,
                   card,
                   retries,
                   update_subscription_card,
                   card_id,
                   card_token,
                   recurrence,
                   capture)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_subscription_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_period_response
import mundiapi.models.get_customer_response
import mundiapi.models.get_card_response
import mundiapi.models.get_subscription_item_response
import mundiapi.models.get_setup_response
import mundiapi.models.get_discount_response

class GetSubscriptionResponse(object):

    """Implementation of the 'GetSubscriptionResponse' model.

    TODO: type model description here.

    Attributes:
        id (string): TODO: type description here.
        code (string): TODO: type description here.
        start_at (datetime): TODO: type description here.
        interval (string): TODO: type description here.
        interval_count (int): TODO: type description here.
        billing_type (string): TODO: type description here.
        current_period (GetPeriodResponse): TODO: type description here.
        payment_method (string): TODO: type description here.
        currency (string): TODO: type description here.
        installments (int): TODO: type description here.
        status (string): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        customer (GetCustomerResponse): TODO: type description here.
        card (GetCardResponse): TODO: type description here.
        items (list of GetSubscriptionItemResponse): TODO: type description
            here.
        statement_descriptor (string): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        setup (GetSetupResponse): TODO: type description here.
        next_billing_at (datetime): TODO: type description here.
        billing_day (int): TODO: type description here.
        minimum_price (int): TODO: type description here.
        canceled_at (datetime): TODO: type description here.
        discounts (list of GetDiscountResponse): Subscription discounts

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "code" : "code",
        "start_at" : "start_at",
        "interval" : "interval",
        "interval_count" : "interval_count",
        "billing_type" : "billing_type",
        "current_period" : "current_period",
        "payment_method" : "payment_method",
        "currency" : "currency",
        "installments" : "installments",
        "status" : "status",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "customer" : "customer",
        "card" : "card",
        "items" : "items",
        "statement_descriptor" : "statement_descriptor",
        "metadata" : "metadata",
        "setup" : "setup",
        "next_billing_at" : "next_billing_at",
        "billing_day" : "billing_day",
        "minimum_price" : "minimum_price",
        "canceled_at" : "canceled_at",
        "discounts" : "discounts"
    }

    def __init__(self,
                 id=None,
                 code=None,
                 start_at=None,
                 interval=None,
                 interval_count=None,
                 billing_type=None,
                 current_period=None,
                 payment_method=None,
                 currency=None,
                 installments=None,
                 status=None,
                 created_at=None,
                 updated_at=None,
                 customer=None,
                 card=None,
                 items=None,
                 statement_descriptor=None,
                 metadata=None,
                 setup=None,
                 next_billing_at=None,
                 billing_day=None,
                 minimum_price=None,
                 canceled_at=None,
                 discounts=None):
        """Constructor for the GetSubscriptionResponse class"""

        # Initialize members of the class
        self.id = id
        self.code = code
        self.start_at = APIHelper.RFC3339DateTime(start_at) if start_at else None
        self.interval = interval
        self.interval_count = interval_count
        self.billing_type = billing_type
        self.current_period = current_period
        self.payment_method = payment_method
        self.currency = currency
        self.installments = installments
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.customer = customer
        self.card = card
        self.items = items
        self.statement_descriptor = statement_descriptor
        self.metadata = metadata
        self.setup = setup
        self.next_billing_at = APIHelper.RFC3339DateTime(next_billing_at) if next_billing_at else None
        self.billing_day = billing_day
        self.minimum_price = minimum_price
        self.canceled_at = APIHelper.RFC3339DateTime(canceled_at) if canceled_at else None
        self.discounts = discounts


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        code = dictionary.get("code")
        start_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("start_at")).datetime if dictionary.get("start_at") else None
        interval = dictionary.get("interval")
        interval_count = dictionary.get("interval_count")
        billing_type = dictionary.get("billing_type")
        current_period = mundiapi.models.get_period_response.GetPeriodResponse.from_dictionary(dictionary.get("current_period")) if dictionary.get("current_period") else None
        payment_method = dictionary.get("payment_method")
        currency = dictionary.get("currency")
        installments = dictionary.get("installments")
        status = dictionary.get("status")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        customer = mundiapi.models.get_customer_response.GetCustomerResponse.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        card = mundiapi.models.get_card_response.GetCardResponse.from_dictionary(dictionary.get("card")) if dictionary.get("card") else None
        items = None
        if dictionary.get("items") != None:
            items = list()
            for structure in dictionary.get("items"):
                items.append(mundiapi.models.get_subscription_item_response.GetSubscriptionItemResponse.from_dictionary(structure))
        statement_descriptor = dictionary.get("statement_descriptor")
        metadata = dictionary.get("metadata")
        setup = mundiapi.models.get_setup_response.GetSetupResponse.from_dictionary(dictionary.get("setup")) if dictionary.get("setup") else None
        next_billing_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("next_billing_at")).datetime if dictionary.get("next_billing_at") else None
        billing_day = dictionary.get("billing_day")
        minimum_price = dictionary.get("minimum_price")
        canceled_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("canceled_at")).datetime if dictionary.get("canceled_at") else None
        discounts = None
        if dictionary.get("discounts") != None:
            discounts = list()
            for structure in dictionary.get("discounts"):
                discounts.append(mundiapi.models.get_discount_response.GetDiscountResponse.from_dictionary(structure))

        # Return an object of this model
        return cls(id,
                   code,
                   start_at,
                   interval,
                   interval_count,
                   billing_type,
                   current_period,
                   payment_method,
                   currency,
                   installments,
                   status,
                   created_at,
                   updated_at,
                   customer,
                   card,
                   items,
                   statement_descriptor,
                   metadata,
                   setup,
                   next_billing_at,
                   billing_day,
                   minimum_price,
                   canceled_at,
                   discounts)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_bank_transfer_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateBankTransferPaymentRequest(object):

    """Implementation of the 'CreateBankTransferPaymentRequest' model.

    Request for creating a bank transfer payment

    Attributes:
        bank (string): Bank
        retries (int): Number of retries

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "bank" : "bank",
        "retries" : "retries"
    }

    def __init__(self,
                 bank=None,
                 retries=None):
        """Constructor for the CreateBankTransferPaymentRequest class"""

        # Initialize members of the class
        self.bank = bank
        self.retries = retries


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        bank = dictionary.get("bank")
        retries = dictionary.get("retries")

        # Return an object of this model
        return cls(bank,
                   retries)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_customer_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_address_response
import mundiapi.models.get_phones_response

class GetCustomerResponse(object):

    """Implementation of the 'GetCustomerResponse' model.

    Response object for getting a customer

    Attributes:
        id (string): TODO: type description here.
        name (string): TODO: type description here.
        email (string): TODO: type description here.
        delinquent (bool): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        document (string): TODO: type description here.
        mtype (string): TODO: type description here.
        fb_access_token (string): TODO: type description here.
        address (GetAddressResponse): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        phones (GetPhonesResponse): TODO: type description here.
        fb_id (long|int): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "name" : "name",
        "email" : "email",
        "delinquent" : "delinquent",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "document" : "document",
        "mtype" : "type",
        "fb_access_token" : "fb_access_token",
        "address" : "address",
        "metadata" : "metadata",
        "phones" : "phones",
        "fb_id" : "fb_id"
    }

    def __init__(self,
                 id=None,
                 name=None,
                 email=None,
                 delinquent=None,
                 created_at=None,
                 updated_at=None,
                 document=None,
                 mtype=None,
                 fb_access_token=None,
                 address=None,
                 metadata=None,
                 phones=None,
                 fb_id=None):
        """Constructor for the GetCustomerResponse class"""

        # Initialize members of the class
        self.id = id
        self.name = name
        self.email = email
        self.delinquent = delinquent
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.document = document
        self.mtype = mtype
        self.fb_access_token = fb_access_token
        self.address = address
        self.metadata = metadata
        self.phones = phones
        self.fb_id = fb_id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        name = dictionary.get("name")
        email = dictionary.get("email")
        delinquent = dictionary.get("delinquent")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        document = dictionary.get("document")
        mtype = dictionary.get("type")
        fb_access_token = dictionary.get("fb_access_token")
        address = mundiapi.models.get_address_response.GetAddressResponse.from_dictionary(dictionary.get("address")) if dictionary.get("address") else None
        metadata = dictionary.get("metadata")
        phones = mundiapi.models.get_phones_response.GetPhonesResponse.from_dictionary(dictionary.get("phones")) if dictionary.get("phones") else None
        fb_id = dictionary.get("fb_id")

        # Return an object of this model
        return cls(id,
                   name,
                   email,
                   delinquent,
                   created_at,
                   updated_at,
                   document,
                   mtype,
                   fb_access_token,
                   address,
                   metadata,
                   phones,
                   fb_id)





# -*- coding: utf-8 -*-

"""
    mundiapi.controllers.subscriptions_controller

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.get_subscription_item_response import GetSubscriptionItemResponse
from ..models.get_usage_response import GetUsageResponse
from ..models.get_subscription_response import GetSubscriptionResponse
from ..models.get_discount_response import GetDiscountResponse
from ..models.list_usages_response import ListUsagesResponse
from ..models.list_subscriptions_response import ListSubscriptionsResponse

class SubscriptionsController(BaseController):

    """A Controller to access Endpoints in the mundiapi API."""


    def update_subscription_item(self,
                                 subscription_id,
                                 item_id,
                                 body):
        """Does a PUT request to /subscriptions/{subscription_id}/items/{item_id}.

        Updates a subscription item

        Args:
            subscription_id (string): Subscription Id
            item_id (string): Item id
            body (UpdateSubscriptionItemRequest): Request for updating a
                subscription item

        Returns:
            GetSubscriptionItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items/{item_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'item_id': item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.put(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionItemResponse.from_dictionary)

    def create_usage(self,
                     subscription_id,
                     item_id,
                     body):
        """Does a POST request to /subscriptions/{subscription_id}/items/{item_id}/usages.

        Creates a usage

        Args:
            subscription_id (string): Subscription Id
            item_id (string): Item id
            body (CreateUsageRequest): Request for creating a usage

        Returns:
            GetUsageResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items/{item_id}/usages'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'item_id': item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetUsageResponse.from_dictionary)

    def update_subscription_billing_date(self,
                                         subscription_id,
                                         request):
        """Does a PATCH request to /subscriptions/{subscription_id}/billing-date.

        Updates the billing date from a subscription

        Args:
            subscription_id (string): The subscription id
            request (UpdateSubscriptionBillingDateRequest): Request for
                updating the subscription billing date

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/billing-date'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def update_subscription_card(self,
                                 subscription_id,
                                 request):
        """Does a PATCH request to /subscriptions/{subscription_id}/card.

        Updates the credit card from a subscription

        Args:
            subscription_id (string): Subscription id
            request (UpdateSubscriptionCardRequest): Request for updating a
                card

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/card'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def create_subscription(self,
                            body):
        """Does a POST request to /subscriptions.

        Creates a new subscription

        Args:
            body (CreateSubscriptionRequest): Request for creating a
                subscription

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions'
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def create_subscription_item(self,
                                 subscription_id,
                                 request):
        """Does a POST request to /subscriptions/{subscription_id}/items.

        Creates a new Subscription item

        Args:
            subscription_id (string): Subscription id
            request (CreateSubscriptionItemRequest): Request for creating a
                subscription item

        Returns:
            GetSubscriptionItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionItemResponse.from_dictionary)

    def create_discount(self,
                        subscription_id,
                        request):
        """Does a POST request to /subscriptions/{subscription_id}/discounts.

        Creates a discount

        Args:
            subscription_id (string): Subscription id
            request (CreateDiscountRequest): Request for creating a discount

        Returns:
            GetDiscountResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/discounts'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetDiscountResponse.from_dictionary)

    def get_subscription(self,
                         subscription_id):
        """Does a GET request to /subscriptions/{subscription_id}.

        Gets a subscription

        Args:
            subscription_id (string): Subscription id

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def update_subscription_payment_method(self,
                                           subscription_id,
                                           request):
        """Does a PATCH request to /subscriptions/{subscription_id}/payment-method.

        Updates the payment method from a subscription

        Args:
            subscription_id (string): Subscription id
            request (UpdateSubscriptionPaymentMethodRequest): Request for
                updating the paymentmethod from a subscription

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/payment-method'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def delete_discount(self,
                        subscription_id,
                        discount_id):
        """Does a DELETE request to /subscriptions/{subscription_id}/discounts/{discount_id}.

        Deletes a discount

        Args:
            subscription_id (string): Subscription id
            discount_id (string): Discount Id

        Returns:
            GetDiscountResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/discounts/{discount_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'discount_id': discount_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetDiscountResponse.from_dictionary)

    def cancel_subscription(self,
                            subscription_id,
                            request=None):
        """Does a DELETE request to /subscriptions/{subscription_id}.

        Cancels a subscription

        Args:
            subscription_id (string): Subscription id
            request (CreateCancelSubscriptionRequest, optional): Request for
                cancelling a subscription

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def delete_subscription_item(self,
                                 subscription_id,
                                 subscription_item_id):
        """Does a DELETE request to /subscriptions/{subscription_id}/items/{subscription_item_id}.

        Deletes a subscription item

        Args:
            subscription_id (string): Subscription id
            subscription_item_id (string): Subscription item id

        Returns:
            GetSubscriptionItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items/{subscription_item_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'subscription_item_id': subscription_item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionItemResponse.from_dictionary)

    def delete_usage(self,
                     subscription_id,
                     item_id,
                     usage_id):
        """Does a DELETE request to /subscriptions/{subscription_id}/items/{item_id}/usages/{usage_id}.

        Deletes a usage

        Args:
            subscription_id (string): The subscription id
            item_id (string): The subscription item id
            usage_id (string): The usage id

        Returns:
            GetUsageResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items/{item_id}/usages/{usage_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'item_id': item_id,
            'usage_id': usage_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetUsageResponse.from_dictionary)

    def get_usages(self,
                   subscription_id,
                   item_id,
                   page=None,
                   size=None):
        """Does a GET request to /subscriptions/{subscription_id}/items/{item_id}/usages.

        Lists all usages from a subscription item

        Args:
            subscription_id (string): The subscription id
            item_id (string): The subscription item id
            page (int, optional): Page number
            size (int, optional): Page size

        Returns:
            ListUsagesResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions/{subscription_id}/items/{item_id}/usages'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id,
            'item_id': item_id
        })
        _query_parameters = {
            'page': page,
            'size': size
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ListUsagesResponse.from_dictionary)

    def update_subscription_metadata(self,
                                     subscription_id,
                                     request):
        """Does a PATCH request to /Subscriptions/{subscription_id}/metadata.

        Updates the metadata from a subscription

        Args:
            subscription_id (string): The subscription id
            request (UpdateMetadataRequest): Request for updating the
                subscrption metadata

        Returns:
            GetSubscriptionResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/Subscriptions/{subscription_id}/metadata'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'subscription_id': subscription_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetSubscriptionResponse.from_dictionary)

    def get_subscriptions(self,
                          page=None,
                          size=None,
                          code=None,
                          billing_type=None,
                          customer_id=None,
                          plan_id=None,
                          card_id=None,
                          status=None,
                          next_billing_since=None,
                          next_billing_until=None,
                          created_since=None,
                          created_until=None):
        """Does a GET request to /subscriptions.

        Gets all subscriptions

        Args:
            page (int, optional): Page number
            size (int, optional): Page size
            code (string, optional): Filter for subscription's code
            billing_type (string, optional): Filter for subscription's billing
                type
            customer_id (string, optional): Filter for subscription's customer
                id
            plan_id (string, optional): Filter for subscription's plan id
            card_id (string, optional): Filter for subscription's card id
            status (string, optional): Filter for subscription's status
            next_billing_since (datetime, optional): Filter for subscription's
                next billing date start range
            next_billing_until (datetime, optional): Filter for subscription's
                next billing date end range
            created_since (datetime, optional): Filter for subscription's
                creation date start range
            created_until (datetime, optional): Filter for subscriptions
                creation date end range

        Returns:
            ListSubscriptionsResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/subscriptions'
        _query_parameters = {
            'page': page,
            'size': size,
            'code': code,
            'billing_type': billing_type,
            'customer_id': customer_id,
            'plan_id': plan_id,
            'card_id': card_id,
            'status': status,
            'next_billing_since': APIHelper.RFC3339DateTime(next_billing_since),
            'next_billing_until': APIHelper.RFC3339DateTime(next_billing_until),
            'created_since': APIHelper.RFC3339DateTime(created_since),
            'created_until': APIHelper.RFC3339DateTime(created_until)
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ListSubscriptionsResponse.from_dictionary)



# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_cancel_charge_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateCancelChargeRequest(object):

    """Implementation of the 'CreateCancelChargeRequest' model.

    Request for canceling a charge.

    Attributes:
        amount (int): The amount that will be canceled.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount" : "amount"
    }

    def __init__(self,
                 amount=None):
        """Constructor for the CreateCancelChargeRequest class"""

        # Initialize members of the class
        self.amount = amount


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = dictionary.get("amount")

        # Return an object of this model
        return cls(amount)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_plan_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_plan_item_response

class GetPlanResponse(object):

    """Implementation of the 'GetPlanResponse' model.

    Response object for getting a plan

    Attributes:
        id (string): TODO: type description here.
        name (string): TODO: type description here.
        description (string): TODO: type description here.
        url (string): TODO: type description here.
        statement_descriptor (string): TODO: type description here.
        interval (string): TODO: type description here.
        interval_count (int): TODO: type description here.
        billing_type (string): TODO: type description here.
        payment_methods (list of string): TODO: type description here.
        installments (list of int): TODO: type description here.
        status (string): TODO: type description here.
        currency (string): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        items (list of GetPlanItemResponse): TODO: type description here.
        billing_days (list of int): TODO: type description here.
        shippable (bool): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        trial_period_days (int): TODO: type description here.
        minimum_price (int): TODO: type description here.
        deleted_at (datetime): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "name" : "name",
        "description" : "description",
        "url" : "url",
        "statement_descriptor" : "statement_descriptor",
        "interval" : "interval",
        "interval_count" : "interval_count",
        "billing_type" : "billing_type",
        "payment_methods" : "payment_methods",
        "installments" : "installments",
        "status" : "status",
        "currency" : "currency",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "items" : "items",
        "billing_days" : "billing_days",
        "shippable" : "shippable",
        "metadata" : "metadata",
        "trial_period_days" : "trial_period_days",
        "minimum_price" : "minimum_price",
        "deleted_at" : "deleted_at"
    }

    def __init__(self,
                 id=None,
                 name=None,
                 description=None,
                 url=None,
                 statement_descriptor=None,
                 interval=None,
                 interval_count=None,
                 billing_type=None,
                 payment_methods=None,
                 installments=None,
                 status=None,
                 currency=None,
                 created_at=None,
                 updated_at=None,
                 items=None,
                 billing_days=None,
                 shippable=None,
                 metadata=None,
                 trial_period_days=None,
                 minimum_price=None,
                 deleted_at=None):
        """Constructor for the GetPlanResponse class"""

        # Initialize members of the class
        self.id = id
        self.name = name
        self.description = description
        self.url = url
        self.statement_descriptor = statement_descriptor
        self.interval = interval
        self.interval_count = interval_count
        self.billing_type = billing_type
        self.payment_methods = payment_methods
        self.installments = installments
        self.status = status
        self.currency = currency
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.items = items
        self.billing_days = billing_days
        self.shippable = shippable
        self.metadata = metadata
        self.trial_period_days = trial_period_days
        self.minimum_price = minimum_price
        self.deleted_at = APIHelper.RFC3339DateTime(deleted_at) if deleted_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        name = dictionary.get("name")
        description = dictionary.get("description")
        url = dictionary.get("url")
        statement_descriptor = dictionary.get("statement_descriptor")
        interval = dictionary.get("interval")
        interval_count = dictionary.get("interval_count")
        billing_type = dictionary.get("billing_type")
        payment_methods = dictionary.get("payment_methods")
        installments = dictionary.get("installments")
        status = dictionary.get("status")
        currency = dictionary.get("currency")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        items = None
        if dictionary.get("items") != None:
            items = list()
            for structure in dictionary.get("items"):
                items.append(mundiapi.models.get_plan_item_response.GetPlanItemResponse.from_dictionary(structure))
        billing_days = dictionary.get("billing_days")
        shippable = dictionary.get("shippable")
        metadata = dictionary.get("metadata")
        trial_period_days = dictionary.get("trial_period_days")
        minimum_price = dictionary.get("minimum_price")
        deleted_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("deleted_at")).datetime if dictionary.get("deleted_at") else None

        # Return an object of this model
        return cls(id,
                   name,
                   description,
                   url,
                   statement_descriptor,
                   interval,
                   interval_count,
                   billing_type,
                   payment_methods,
                   installments,
                   status,
                   currency,
                   created_at,
                   updated_at,
                   items,
                   billing_days,
                   shippable,
                   metadata,
                   trial_period_days,
                   minimum_price,
                   deleted_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_order_item_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_seller_request

class CreateOrderItemRequest(object):

    """Implementation of the 'CreateOrderItemRequest' model.

    Request for creating an order item

    Attributes:
        amount (int): Amount
        description (string): Description
        quantity (int): Quantity
        seller (CreateSellerRequest): Item seller
        seller_id (string): seller identificator

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount" : "amount",
        "description" : "description",
        "quantity" : "quantity",
        "seller" : "seller",
        "seller_id" : "seller_id"
    }

    def __init__(self,
                 amount=None,
                 description=None,
                 quantity=None,
                 seller=None,
                 seller_id=None):
        """Constructor for the CreateOrderItemRequest class"""

        # Initialize members of the class
        self.amount = amount
        self.description = description
        self.quantity = quantity
        self.seller = seller
        self.seller_id = seller_id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = dictionary.get("amount")
        description = dictionary.get("description")
        quantity = dictionary.get("quantity")
        seller = mundiapi.models.create_seller_request.CreateSellerRequest.from_dictionary(dictionary.get("seller")) if dictionary.get("seller") else None
        seller_id = dictionary.get("seller_id")

        # Return an object of this model
        return cls(amount,
                   description,
                   quantity,
                   seller,
                   seller_id)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_charge_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_transaction_response
import mundiapi.models.get_invoice_response
import mundiapi.models.get_order_response
import mundiapi.models.get_customer_response

class GetChargeResponse(object):

    """Implementation of the 'GetChargeResponse' model.

    Response object for getting a charge

    Attributes:
        id (string): TODO: type description here.
        code (string): TODO: type description here.
        gateway_id (string): TODO: type description here.
        amount (int): TODO: type description here.
        status (string): TODO: type description here.
        currency (string): TODO: type description here.
        payment_method (string): TODO: type description here.
        due_at (datetime): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        last_transaction (GetTransactionResponse): TODO: type description
            here.
        invoice (GetInvoiceResponse): TODO: type description here.
        order (GetOrderResponse): TODO: type description here.
        customer (GetCustomerResponse): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        paid_at (datetime): TODO: type description here.
        canceled_at (datetime): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "code" : "code",
        "gateway_id" : "gateway_id",
        "amount" : "amount",
        "status" : "status",
        "currency" : "currency",
        "payment_method" : "payment_method",
        "due_at" : "due_at",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "last_transaction" : "last_transaction",
        "invoice" : "invoice",
        "order" : "order",
        "customer" : "customer",
        "metadata" : "metadata",
        "paid_at" : "paid_at",
        "canceled_at" : "canceled_at"
    }

    def __init__(self,
                 id=None,
                 code=None,
                 gateway_id=None,
                 amount=None,
                 status=None,
                 currency=None,
                 payment_method=None,
                 due_at=None,
                 created_at=None,
                 updated_at=None,
                 last_transaction=None,
                 invoice=None,
                 order=None,
                 customer=None,
                 metadata=None,
                 paid_at=None,
                 canceled_at=None):
        """Constructor for the GetChargeResponse class"""

        # Initialize members of the class
        self.id = id
        self.code = code
        self.gateway_id = gateway_id
        self.amount = amount
        self.status = status
        self.currency = currency
        self.payment_method = payment_method
        self.due_at = APIHelper.RFC3339DateTime(due_at) if due_at else None
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.last_transaction = last_transaction
        self.invoice = invoice
        self.order = order
        self.customer = customer
        self.metadata = metadata
        self.paid_at = APIHelper.RFC3339DateTime(paid_at) if paid_at else None
        self.canceled_at = APIHelper.RFC3339DateTime(canceled_at) if canceled_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        code = dictionary.get("code")
        gateway_id = dictionary.get("gateway_id")
        amount = dictionary.get("amount")
        status = dictionary.get("status")
        currency = dictionary.get("currency")
        payment_method = dictionary.get("payment_method")
        due_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("due_at")).datetime if dictionary.get("due_at") else None
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        last_transaction = mundiapi.models.get_transaction_response.GetTransactionResponse.from_dictionary(dictionary.get("last_transaction")) if dictionary.get("last_transaction") else None
        invoice = mundiapi.models.get_invoice_response.GetInvoiceResponse.from_dictionary(dictionary.get("invoice")) if dictionary.get("invoice") else None
        order = mundiapi.models.get_order_response.GetOrderResponse.from_dictionary(dictionary.get("order")) if dictionary.get("order") else None
        customer = mundiapi.models.get_customer_response.GetCustomerResponse.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        metadata = dictionary.get("metadata")
        paid_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("paid_at")).datetime if dictionary.get("paid_at") else None
        canceled_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("canceled_at")).datetime if dictionary.get("canceled_at") else None

        # Return an object of this model
        return cls(id,
                   code,
                   gateway_id,
                   amount,
                   status,
                   currency,
                   payment_method,
                   due_at,
                   created_at,
                   updated_at,
                   last_transaction,
                   invoice,
                   order,
                   customer,
                   metadata,
                   paid_at,
                   canceled_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_card_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_address_request
import mundiapi.models.create_card_options_request

class CreateCardRequest(object):

    """Implementation of the 'CreateCardRequest' model.

    Card data

    Attributes:
        number (string): Credit card number
        holder_name (string): Holder name, as written on the card
        exp_month (int): The expiration month
        exp_year (int): The expiration year, that can be informed with 2 or 4
            digits
        cvv (string): The card's security code
        billing_address (CreateAddressRequest): Card's billing address
        brand (string): Card brand
        billing_address_id (string): The address id for the billing address
        metadata (dict<object, string>): Metadata
        mtype (string): Card type
        options (CreateCardOptionsRequest): Options for creating the card
        holder_document (string): Document number for the card's holder

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "number" : "number",
        "holder_name" : "holder_name",
        "exp_month" : "exp_month",
        "exp_year" : "exp_year",
        "cvv" : "cvv",
        "billing_address" : "billing_address",
        "brand" : "brand",
        "billing_address_id" : "billing_address_id",
        "metadata" : "metadata",
        "mtype" : "type",
        "options" : "options",
        "holder_document" : "holder_document"
    }

    def __init__(self,
                 number=None,
                 holder_name=None,
                 exp_month=None,
                 exp_year=None,
                 cvv=None,
                 billing_address=None,
                 brand=None,
                 billing_address_id=None,
                 metadata=None,
                 mtype='credit',
                 options=None,
                 holder_document=None):
        """Constructor for the CreateCardRequest class"""

        # Initialize members of the class
        self.number = number
        self.holder_name = holder_name
        self.exp_month = exp_month
        self.exp_year = exp_year
        self.cvv = cvv
        self.billing_address = billing_address
        self.brand = brand
        self.billing_address_id = billing_address_id
        self.metadata = metadata
        self.mtype = mtype
        self.options = options
        self.holder_document = holder_document


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        number = dictionary.get("number")
        holder_name = dictionary.get("holder_name")
        exp_month = dictionary.get("exp_month")
        exp_year = dictionary.get("exp_year")
        cvv = dictionary.get("cvv")
        billing_address = mundiapi.models.create_address_request.CreateAddressRequest.from_dictionary(dictionary.get("billing_address")) if dictionary.get("billing_address") else None
        brand = dictionary.get("brand")
        billing_address_id = dictionary.get("billing_address_id")
        metadata = dictionary.get("metadata")
        mtype = dictionary.get("type") if dictionary.get("type") else 'credit'
        options = mundiapi.models.create_card_options_request.CreateCardOptionsRequest.from_dictionary(dictionary.get("options")) if dictionary.get("options") else None
        holder_document = dictionary.get("holder_document")

        # Return an object of this model
        return cls(number,
                   holder_name,
                   exp_month,
                   exp_year,
                   cvv,
                   billing_address,
                   brand,
                   billing_address_id,
                   metadata,
                   mtype,
                   options,
                   holder_document)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_discount_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateDiscountRequest(object):

    """Implementation of the 'CreateDiscountRequest' model.

    Request for creating a new discount

    Attributes:
        value (float): The discount value
        discount_type (string): Discount type. Can be either flat or
            percentage.
        item_id (string): The item where the discount will be applied
        cycles (int): Number of cycles that the discount will be applied

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "value" : "value",
        "discount_type" : "discount_type",
        "item_id" : "item_id",
        "cycles" : "cycles"
    }

    def __init__(self,
                 value=None,
                 discount_type=None,
                 item_id=None,
                 cycles=None):
        """Constructor for the CreateDiscountRequest class"""

        # Initialize members of the class
        self.value = value
        self.discount_type = discount_type
        self.item_id = item_id
        self.cycles = cycles


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        value = dictionary.get("value")
        discount_type = dictionary.get("discount_type")
        item_id = dictionary.get("item_id")
        cycles = dictionary.get("cycles")

        # Return an object of this model
        return cls(value,
                   discount_type,
                   item_id,
                   cycles)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_seller_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_card_request

class CreateSellerRequest(object):

    """Implementation of the 'CreateSellerRequest' model.

    TODO: type model description here.

    Attributes:
        name (string): Name
        code (string): Seller's code identification
        description (string): Description
        document (string): Document number (individual / company)
        address (string): Address
        mtype (string): Person type (individual / company)
        metadata (CreateCardRequest): Metadata

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name" : "name",
        "code" : "code",
        "description" : "description",
        "document" : "document",
        "address" : "address",
        "mtype" : "type",
        "metadata" : "metadata"
    }

    def __init__(self,
                 name=None,
                 code=None,
                 description=None,
                 document=None,
                 address=None,
                 mtype=None,
                 metadata=None):
        """Constructor for the CreateSellerRequest class"""

        # Initialize members of the class
        self.name = name
        self.code = code
        self.description = description
        self.document = document
        self.address = address
        self.mtype = mtype
        self.metadata = metadata


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name")
        code = dictionary.get("code")
        description = dictionary.get("description")
        document = dictionary.get("document")
        address = dictionary.get("address")
        mtype = dictionary.get("type")
        metadata = mundiapi.models.create_card_request.CreateCardRequest.from_dictionary(dictionary.get("metadata")) if dictionary.get("metadata") else None

        # Return an object of this model
        return cls(name,
                   code,
                   description,
                   document,
                   address,
                   mtype,
                   metadata)





# -*- coding: utf-8 -*-

"""
    mundiapi.controllers.invoices_controller

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.get_invoice_response import GetInvoiceResponse
from ..models.list_invoices_response import ListInvoicesResponse

class InvoicesController(BaseController):

    """A Controller to access Endpoints in the mundiapi API."""


    def get_invoice(self,
                    invoice_id):
        """Does a GET request to /invoices/{invoice_id}.

        Gets an invoice

        Args:
            invoice_id (string): Invoice Id

        Returns:
            GetInvoiceResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/invoices/{invoice_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'invoice_id': invoice_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetInvoiceResponse.from_dictionary)

    def cancel_invoice(self,
                       invoice_id):
        """Does a DELETE request to /invoices/{invoice_id}.

        Cancels an invoice

        Args:
            invoice_id (string): Invoice id

        Returns:
            GetInvoiceResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/invoices/{invoice_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'invoice_id': invoice_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetInvoiceResponse.from_dictionary)

    def update_invoice_metadata(self,
                                invoice_id,
                                request):
        """Does a PATCH request to /invoices/{invoice_id}/metadata.

        Updates the metadata from an invoice

        Args:
            invoice_id (string): The invoice id
            request (UpdateMetadataRequest): Request for updating the invoice
                metadata

        Returns:
            GetInvoiceResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/invoices/{invoice_id}/metadata'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'invoice_id': invoice_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetInvoiceResponse.from_dictionary)

    def get_invoices(self,
                     page=None,
                     size=None,
                     code=None,
                     customer_id=None,
                     subscription_id=None,
                     created_since=None,
                     created_until=None,
                     status=None,
                     due_since=None,
                     due_until=None):
        """Does a GET request to /invoices.

        Gets all invoices

        Args:
            page (int, optional): Page number
            size (int, optional): Page size
            code (string, optional): Filter for Invoice's code
            customer_id (string, optional): Filter for Invoice's customer id
            subscription_id (string, optional): Filter for Invoice's
                subscription id
            created_since (datetime, optional): Filter for Invoice's creation
                date start range
            created_until (datetime, optional): Filter for Invoices creation
                date end range
            status (string, optional): Filter for Invoice's status
            due_since (datetime, optional): Filter for Invoice's due date
                start range
            due_until (datetime, optional): Filter for Invoice's due date end
                range

        Returns:
            ListInvoicesResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/invoices'
        _query_parameters = {
            'page': page,
            'size': size,
            'code': code,
            'customer_id': customer_id,
            'subscription_id': subscription_id,
            'created_since': APIHelper.RFC3339DateTime(created_since),
            'created_until': APIHelper.RFC3339DateTime(created_until),
            'status': status,
            'due_since': APIHelper.RFC3339DateTime(due_since),
            'due_until': APIHelper.RFC3339DateTime(due_until)
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ListInvoicesResponse.from_dictionary)



# -*- coding: utf-8 -*-

"""
    mundiapi.models.update_charge_payment_method_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_credit_card_payment_request
import mundiapi.models.create_boleto_payment_request
import mundiapi.models.create_voucher_payment_request

class UpdateChargePaymentMethodRequest(object):

    """Implementation of the 'UpdateChargePaymentMethodRequest' model.

    Request for updating the payment method of a charge

    Attributes:
        update_subscription (bool): Indicates if the payment method from the
            subscription must also be updated
        payment_method (string): The new payment method
        credit_card (CreateCreditCardPaymentRequest): Credit card data
        boleto (CreateBoletoPaymentRequest): Boleto data
        voucher (CreateVoucherPaymentRequest): Voucher data

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "update_subscription" : "update_subscription",
        "payment_method" : "payment_method",
        "credit_card" : "credit_card",
        "boleto" : "boleto",
        "voucher" : "voucher"
    }

    def __init__(self,
                 update_subscription=None,
                 payment_method=None,
                 credit_card=None,
                 boleto=None,
                 voucher=None):
        """Constructor for the UpdateChargePaymentMethodRequest class"""

        # Initialize members of the class
        self.update_subscription = update_subscription
        self.payment_method = payment_method
        self.credit_card = credit_card
        self.boleto = boleto
        self.voucher = voucher


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        update_subscription = dictionary.get("update_subscription")
        payment_method = dictionary.get("payment_method")
        credit_card = mundiapi.models.create_credit_card_payment_request.CreateCreditCardPaymentRequest.from_dictionary(dictionary.get("credit_card")) if dictionary.get("credit_card") else None
        boleto = mundiapi.models.create_boleto_payment_request.CreateBoletoPaymentRequest.from_dictionary(dictionary.get("boleto")) if dictionary.get("boleto") else None
        voucher = mundiapi.models.create_voucher_payment_request.CreateVoucherPaymentRequest.from_dictionary(dictionary.get("voucher")) if dictionary.get("voucher") else None

        # Return an object of this model
        return cls(update_subscription,
                   payment_method,
                   credit_card,
                   boleto,
                   voucher)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.update_subscription_billing_date_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper

class UpdateSubscriptionBillingDateRequest(object):

    """Implementation of the 'UpdateSubscriptionBillingDateRequest' model.

    Request for updating the due date from a subscription

    Attributes:
        next_billing_at (datetime): The date when the next subscription
            billing must occur

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "next_billing_at" : "next_billing_at"
    }

    def __init__(self,
                 next_billing_at=None):
        """Constructor for the UpdateSubscriptionBillingDateRequest class"""

        # Initialize members of the class
        self.next_billing_at = APIHelper.RFC3339DateTime(next_billing_at) if next_billing_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        next_billing_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("next_billing_at")).datetime if dictionary.get("next_billing_at") else None

        # Return an object of this model
        return cls(next_billing_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_invoice_item_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.get_pricing_scheme_response
import mundiapi.models.get_price_bracket_response

class GetInvoiceItemResponse(object):

    """Implementation of the 'GetInvoiceItemResponse' model.

    Response object for getting an invoice item

    Attributes:
        amount (int): TODO: type description here.
        description (string): TODO: type description here.
        pricing_scheme (GetPricingSchemeResponse): TODO: type description
            here.
        price_bracket (GetPriceBracketResponse): TODO: type description here.
        quantity (int): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount" : "amount",
        "description" : "description",
        "pricing_scheme" : "pricing_scheme",
        "price_bracket" : "price_bracket",
        "quantity" : "quantity"
    }

    def __init__(self,
                 amount=None,
                 description=None,
                 pricing_scheme=None,
                 price_bracket=None,
                 quantity=None):
        """Constructor for the GetInvoiceItemResponse class"""

        # Initialize members of the class
        self.amount = amount
        self.description = description
        self.pricing_scheme = pricing_scheme
        self.price_bracket = price_bracket
        self.quantity = quantity


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = dictionary.get("amount")
        description = dictionary.get("description")
        pricing_scheme = mundiapi.models.get_pricing_scheme_response.GetPricingSchemeResponse.from_dictionary(dictionary.get("pricing_scheme")) if dictionary.get("pricing_scheme") else None
        price_bracket = mundiapi.models.get_price_bracket_response.GetPriceBracketResponse.from_dictionary(dictionary.get("price_bracket")) if dictionary.get("price_bracket") else None
        quantity = dictionary.get("quantity")

        # Return an object of this model
        return cls(amount,
                   description,
                   pricing_scheme,
                   price_bracket,
                   quantity)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_plan_item_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_pricing_scheme_response
import mundiapi.models.get_plan_response

class GetPlanItemResponse(object):

    """Implementation of the 'GetPlanItemResponse' model.

    Response object for getting a plan item

    Attributes:
        id (string): TODO: type description here.
        name (string): TODO: type description here.
        status (string): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        pricing_scheme (GetPricingSchemeResponse): TODO: type description
            here.
        description (string): TODO: type description here.
        plan (GetPlanResponse): TODO: type description here.
        quantity (int): TODO: type description here.
        cycles (int): TODO: type description here.
        deleted_at (datetime): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "name" : "name",
        "status" : "status",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "pricing_scheme" : "pricing_scheme",
        "description" : "description",
        "plan" : "plan",
        "quantity" : "quantity",
        "cycles" : "cycles",
        "deleted_at" : "deleted_at"
    }

    def __init__(self,
                 id=None,
                 name=None,
                 status=None,
                 created_at=None,
                 updated_at=None,
                 pricing_scheme=None,
                 description=None,
                 plan=None,
                 quantity=None,
                 cycles=None,
                 deleted_at=None):
        """Constructor for the GetPlanItemResponse class"""

        # Initialize members of the class
        self.id = id
        self.name = name
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.pricing_scheme = pricing_scheme
        self.description = description
        self.plan = plan
        self.quantity = quantity
        self.cycles = cycles
        self.deleted_at = APIHelper.RFC3339DateTime(deleted_at) if deleted_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        name = dictionary.get("name")
        status = dictionary.get("status")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        pricing_scheme = mundiapi.models.get_pricing_scheme_response.GetPricingSchemeResponse.from_dictionary(dictionary.get("pricing_scheme")) if dictionary.get("pricing_scheme") else None
        description = dictionary.get("description")
        plan = mundiapi.models.get_plan_response.GetPlanResponse.from_dictionary(dictionary.get("plan")) if dictionary.get("plan") else None
        quantity = dictionary.get("quantity")
        cycles = dictionary.get("cycles")
        deleted_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("deleted_at")).datetime if dictionary.get("deleted_at") else None

        # Return an object of this model
        return cls(id,
                   name,
                   status,
                   created_at,
                   updated_at,
                   pricing_scheme,
                   description,
                   plan,
                   quantity,
                   cycles,
                   deleted_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_checkout_boleto_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper

class CreateCheckoutBoletoPaymentRequest(object):

    """Implementation of the 'CreateCheckoutBoletoPaymentRequest' model.

    TODO: type model description here.

    Attributes:
        bank (string): Bank identifier
        instructions (string): Instructions
        due_at (datetime): Due date

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "bank" : "bank",
        "instructions" : "instructions",
        "due_at" : "due_at"
    }

    def __init__(self,
                 bank=None,
                 instructions=None,
                 due_at=None):
        """Constructor for the CreateCheckoutBoletoPaymentRequest class"""

        # Initialize members of the class
        self.bank = bank
        self.instructions = instructions
        self.due_at = APIHelper.RFC3339DateTime(due_at) if due_at else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        bank = dictionary.get("bank")
        instructions = dictionary.get("instructions")
        due_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("due_at")).datetime if dictionary.get("due_at") else None

        # Return an object of this model
        return cls(bank,
                   instructions,
                   due_at)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.update_plan_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class UpdatePlanRequest(object):

    """Implementation of the 'UpdatePlanRequest' model.

    Request for updating a plan

    Attributes:
        name (string): Plan's name
        description (string): Description
        installments (list of int): Number os installments
        statement_descriptor (string): Text that will be shown on the credit
            card's statement
        currency (string): Currency
        interval (string): Interval
        interval_count (int): Interval count
        payment_methods (list of string): Payment methods accepted by the
            plan
        billing_type (string): Billing type
        status (string): Plan status
        shippable (bool): Indicates if the plan is shippable
        billing_days (list of int): Billing days accepted by the plan
        metadata (dict<object, string>): Metadata
        minimum_price (int): Minimum price
        trial_period_days (int): Number of trial period in days, where the
            customer will not be charged

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name" : "name",
        "description" : "description",
        "installments" : "installments",
        "statement_descriptor" : "statement_descriptor",
        "currency" : "currency",
        "interval" : "interval",
        "interval_count" : "interval_count",
        "payment_methods" : "payment_methods",
        "billing_type" : "billing_type",
        "status" : "status",
        "shippable" : "shippable",
        "billing_days" : "billing_days",
        "metadata" : "metadata",
        "minimum_price" : "minimum_price",
        "trial_period_days" : "trial_period_days"
    }

    def __init__(self,
                 name=None,
                 description=None,
                 installments=None,
                 statement_descriptor=None,
                 currency=None,
                 interval=None,
                 interval_count=None,
                 payment_methods=None,
                 billing_type=None,
                 status=None,
                 shippable=None,
                 billing_days=None,
                 metadata=None,
                 minimum_price=None,
                 trial_period_days=None):
        """Constructor for the UpdatePlanRequest class"""

        # Initialize members of the class
        self.name = name
        self.description = description
        self.installments = installments
        self.statement_descriptor = statement_descriptor
        self.currency = currency
        self.interval = interval
        self.interval_count = interval_count
        self.payment_methods = payment_methods
        self.billing_type = billing_type
        self.status = status
        self.shippable = shippable
        self.billing_days = billing_days
        self.metadata = metadata
        self.minimum_price = minimum_price
        self.trial_period_days = trial_period_days


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name")
        description = dictionary.get("description")
        installments = dictionary.get("installments")
        statement_descriptor = dictionary.get("statement_descriptor")
        currency = dictionary.get("currency")
        interval = dictionary.get("interval")
        interval_count = dictionary.get("interval_count")
        payment_methods = dictionary.get("payment_methods")
        billing_type = dictionary.get("billing_type")
        status = dictionary.get("status")
        shippable = dictionary.get("shippable")
        billing_days = dictionary.get("billing_days")
        metadata = dictionary.get("metadata")
        minimum_price = dictionary.get("minimum_price")
        trial_period_days = dictionary.get("trial_period_days")

        # Return an object of this model
        return cls(name,
                   description,
                   installments,
                   statement_descriptor,
                   currency,
                   interval,
                   interval_count,
                   payment_methods,
                   billing_type,
                   status,
                   shippable,
                   billing_days,
                   metadata,
                   minimum_price,
                   trial_period_days)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_order_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.get_order_item_response
import mundiapi.models.get_customer_response
import mundiapi.models.get_charge_response
import mundiapi.models.get_shipping_response
import mundiapi.models.get_checkout_payment_settings_response

class GetOrderResponse(object):

    """Implementation of the 'GetOrderResponse' model.

    Response object for getting an Order

    Attributes:
        id (string): TODO: type description here.
        code (string): TODO: type description here.
        currency (string): TODO: type description here.
        items (list of GetOrderItemResponse): TODO: type description here.
        customer (GetCustomerResponse): TODO: type description here.
        status (string): TODO: type description here.
        created_at (datetime): TODO: type description here.
        updated_at (datetime): TODO: type description here.
        charges (list of GetChargeResponse): TODO: type description here.
        invoice_url (string): TODO: type description here.
        shipping (GetShippingResponse): TODO: type description here.
        metadata (dict<object, string>): TODO: type description here.
        checkouts (list of GetCheckoutPaymentSettingsResponse): Checkout
            Payment Settings Response

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id" : "id",
        "code" : "code",
        "currency" : "currency",
        "items" : "items",
        "customer" : "customer",
        "status" : "status",
        "created_at" : "created_at",
        "updated_at" : "updated_at",
        "charges" : "charges",
        "invoice_url" : "invoice_url",
        "shipping" : "shipping",
        "metadata" : "metadata",
        "checkouts" : "checkouts"
    }

    def __init__(self,
                 id=None,
                 code=None,
                 currency=None,
                 items=None,
                 customer=None,
                 status=None,
                 created_at=None,
                 updated_at=None,
                 charges=None,
                 invoice_url=None,
                 shipping=None,
                 metadata=None,
                 checkouts=None):
        """Constructor for the GetOrderResponse class"""

        # Initialize members of the class
        self.id = id
        self.code = code
        self.currency = currency
        self.items = items
        self.customer = customer
        self.status = status
        self.created_at = APIHelper.RFC3339DateTime(created_at) if created_at else None
        self.updated_at = APIHelper.RFC3339DateTime(updated_at) if updated_at else None
        self.charges = charges
        self.invoice_url = invoice_url
        self.shipping = shipping
        self.metadata = metadata
        self.checkouts = checkouts


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id")
        code = dictionary.get("code")
        currency = dictionary.get("currency")
        items = None
        if dictionary.get("items") != None:
            items = list()
            for structure in dictionary.get("items"):
                items.append(mundiapi.models.get_order_item_response.GetOrderItemResponse.from_dictionary(structure))
        customer = mundiapi.models.get_customer_response.GetCustomerResponse.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        status = dictionary.get("status")
        created_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("created_at")).datetime if dictionary.get("created_at") else None
        updated_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("updated_at")).datetime if dictionary.get("updated_at") else None
        charges = None
        if dictionary.get("charges") != None:
            charges = list()
            for structure in dictionary.get("charges"):
                charges.append(mundiapi.models.get_charge_response.GetChargeResponse.from_dictionary(structure))
        invoice_url = dictionary.get("invoice_url")
        shipping = mundiapi.models.get_shipping_response.GetShippingResponse.from_dictionary(dictionary.get("shipping")) if dictionary.get("shipping") else None
        metadata = dictionary.get("metadata")
        checkouts = None
        if dictionary.get("checkouts") != None:
            checkouts = list()
            for structure in dictionary.get("checkouts"):
                checkouts.append(mundiapi.models.get_checkout_payment_settings_response.GetCheckoutPaymentSettingsResponse.from_dictionary(structure))

        # Return an object of this model
        return cls(id,
                   code,
                   currency,
                   items,
                   customer,
                   status,
                   created_at,
                   updated_at,
                   charges,
                   invoice_url,
                   shipping,
                   metadata,
                   checkouts)





__all__ = [
    'base_controller',
    'charges_controller',
    'customers_controller',
    'invoices_controller',
    'plans_controller',
    'subscriptions_controller',
    'orders_controller',
    'tokens_controller',
]


# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_voucher_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_card_request

class CreateVoucherPaymentRequest(object):

    """Implementation of the 'CreateVoucherPaymentRequest' model.

    The settings for creating a voucher payment

    Attributes:
        capture (bool): Indicates if the operation should be only
            authorization or auth and capture.
        statement_descriptor (string): The text that will be shown on the
            voucher's statement
        card_id (string): Card id
        card_token (string): Card token
        card (CreateCardRequest): Card info

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "capture" : "capture",
        "statement_descriptor" : "statement_descriptor",
        "card_id" : "card_id",
        "card_token" : "card_token",
        "card" : "Card"
    }

    def __init__(self,
                 capture=None,
                 statement_descriptor=None,
                 card_id=None,
                 card_token=None,
                 card=None):
        """Constructor for the CreateVoucherPaymentRequest class"""

        # Initialize members of the class
        self.capture = capture
        self.statement_descriptor = statement_descriptor
        self.card_id = card_id
        self.card_token = card_token
        self.card = card


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        capture = dictionary.get("capture")
        statement_descriptor = dictionary.get("statement_descriptor")
        card_id = dictionary.get("card_id")
        card_token = dictionary.get("card_token")
        card = mundiapi.models.create_card_request.CreateCardRequest.from_dictionary(dictionary.get("Card")) if dictionary.get("Card") else None

        # Return an object of this model
        return cls(capture,
                   statement_descriptor,
                   card_id,
                   card_token,
                   card)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.get_checkout_payment_settings_response

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.get_customer_response

class GetCheckoutPaymentSettingsResponse(object):

    """Implementation of the 'GetCheckoutPaymentSettingsResponse' model.

    Checkout Payment Settings Response

    Attributes:
        success_url (string): Success Url
        payment_url (string): Payment Url
        accepted_payment_methods (list of string): Accepted Payment Methods
        status (string): Status
        customer (GetCustomerResponse): Customer
        amount (int): Payment amount
        default_payment_method (string): Default Payment Method
        gateway_affiliation_id (string): Gateway Affiliation Id

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "success_url" : "success_url",
        "payment_url" : "payment_url",
        "accepted_payment_methods" : "accepted_payment_methods",
        "status" : "status",
        "customer" : "customer",
        "amount" : "amount",
        "default_payment_method" : "default_payment_method",
        "gateway_affiliation_id" : "gateway_affiliation_id"
    }

    def __init__(self,
                 success_url=None,
                 payment_url=None,
                 accepted_payment_methods=None,
                 status=None,
                 customer=None,
                 amount=None,
                 default_payment_method=None,
                 gateway_affiliation_id=None):
        """Constructor for the GetCheckoutPaymentSettingsResponse class"""

        # Initialize members of the class
        self.success_url = success_url
        self.payment_url = payment_url
        self.accepted_payment_methods = accepted_payment_methods
        self.status = status
        self.customer = customer
        self.amount = amount
        self.default_payment_method = default_payment_method
        self.gateway_affiliation_id = gateway_affiliation_id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        success_url = dictionary.get("success_url")
        payment_url = dictionary.get("payment_url")
        accepted_payment_methods = dictionary.get("accepted_payment_methods")
        status = dictionary.get("status")
        customer = mundiapi.models.get_customer_response.GetCustomerResponse.from_dictionary(dictionary.get("customer")) if dictionary.get("customer") else None
        amount = dictionary.get("amount")
        default_payment_method = dictionary.get("default_payment_method")
        gateway_affiliation_id = dictionary.get("gateway_affiliation_id")

        # Return an object of this model
        return cls(success_url,
                   payment_url,
                   accepted_payment_methods,
                   status,
                   customer,
                   amount,
                   default_payment_method,
                   gateway_affiliation_id)





# -*- coding: utf-8 -*-

"""
    mundiapi.controllers.plans_controller

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from .base_controller import BaseController
from ..api_helper import APIHelper
from ..configuration import Configuration
from ..http.auth.basic_auth import BasicAuth
from ..models.get_plan_item_response import GetPlanItemResponse
from ..models.get_plan_response import GetPlanResponse
from ..models.list_plans_response import ListPlansResponse

class PlansController(BaseController):

    """A Controller to access Endpoints in the mundiapi API."""


    def update_plan_item(self,
                         plan_id,
                         plan_item_id,
                         body):
        """Does a PUT request to /plans/{plan_id}/items/{plan_item_id}.

        Updates a plan item

        Args:
            plan_id (string): Plan id
            plan_item_id (string): Plan item id
            body (UpdatePlanItemRequest): Request for updating the plan item

        Returns:
            GetPlanItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}/items/{plan_item_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id,
            'plan_item_id': plan_item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.put(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanItemResponse.from_dictionary)

    def get_plan(self,
                 plan_id):
        """Does a GET request to /plans/{plan_id}.

        Gets a plan

        Args:
            plan_id (string): Plan id

        Returns:
            GetPlanResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanResponse.from_dictionary)

    def create_plan_item(self,
                         plan_id,
                         request):
        """Does a POST request to /plans/{plan_id}/items.

        Adds a new item to a plan

        Args:
            plan_id (string): Plan id
            request (CreatePlanItemRequest): Request for creating a plan item

        Returns:
            GetPlanItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}/items'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanItemResponse.from_dictionary)

    def update_plan(self,
                    plan_id,
                    request):
        """Does a PUT request to /plans/{plan_id}.

        Updates a plan

        Args:
            plan_id (string): Plan id
            request (UpdatePlanRequest): Request for updating a plan

        Returns:
            GetPlanResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.put(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanResponse.from_dictionary)

    def create_plan(self,
                    body):
        """Does a POST request to /plans.

        Creates a new plan

        Args:
            body (CreatePlanRequest): Request for creating a plan

        Returns:
            GetPlanResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans'
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=APIHelper.json_serialize(body))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanResponse.from_dictionary)

    def delete_plan(self,
                    plan_id):
        """Does a DELETE request to /plans/{plan_id}.

        Deletes a plan

        Args:
            plan_id (string): Plan id

        Returns:
            GetPlanResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanResponse.from_dictionary)

    def get_plan_item(self,
                      plan_id,
                      plan_item_id):
        """Does a GET request to /plans/{plan_id}/items/{plan_item_id}.

        Gets a plan item

        Args:
            plan_id (string): Plan id
            plan_item_id (string): Plan item id

        Returns:
            GetPlanItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}/items/{plan_item_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id,
            'plan_item_id': plan_item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanItemResponse.from_dictionary)

    def delete_plan_item(self,
                         plan_id,
                         plan_item_id):
        """Does a DELETE request to /plans/{plan_id}/items/{plan_item_id}.

        Removes an item from a plan

        Args:
            plan_id (string): Plan id
            plan_item_id (string): Plan item id

        Returns:
            GetPlanItemResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans/{plan_id}/items/{plan_item_id}'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id,
            'plan_item_id': plan_item_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.delete(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanItemResponse.from_dictionary)

    def update_plan_metadata(self,
                             plan_id,
                             request):
        """Does a PATCH request to /Plans/{plan_id}/metadata.

        Updates the metadata from a plan

        Args:
            plan_id (string): The plan id
            request (UpdateMetadataRequest): Request for updating the plan
                metadata

        Returns:
            GetPlanResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/Plans/{plan_id}/metadata'
        _query_builder = APIHelper.append_url_with_template_parameters(_query_builder, { 
            'plan_id': plan_id
        })
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'content-type': 'application/json; charset=utf-8'
        }

        # Prepare and execute request
        _request = self.http_client.patch(_query_url, headers=_headers, parameters=APIHelper.json_serialize(request))
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, GetPlanResponse.from_dictionary)

    def get_plans(self,
                  page=None,
                  size=None,
                  name=None,
                  status=None,
                  billing_type=None,
                  created_since=None,
                  created_until=None):
        """Does a GET request to /plans.

        Gets all plans

        Args:
            page (int, optional): Page number
            size (int, optional): Page size
            name (string, optional): Filter for Plan's name
            status (string, optional): Filter for Plan's status
            billing_type (string, optional): Filter for plan's billing type
            created_since (datetime, optional): Filter for plan's creation
                date start range
            created_until (datetime, optional): Filter for plan's creation
                date end range

        Returns:
            ListPlansResponse: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _query_builder = Configuration.base_uri
        _query_builder += '/plans'
        _query_parameters = {
            'page': page,
            'size': size,
            'name': name,
            'status': status,
            'billing_type': billing_type,
            'created_since': APIHelper.RFC3339DateTime(created_since),
            'created_until': APIHelper.RFC3339DateTime(created_until)
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        BasicAuth.apply(_request)
        _context = self.execute_request(_request)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ListPlansResponse.from_dictionary)



# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_customer_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_address_request
import mundiapi.models.create_phones_request

class CreateCustomerRequest(object):

    """Implementation of the 'CreateCustomerRequest' model.

    Request for creating a new customer

    Attributes:
        name (string): Name
        email (string): Email
        document (string): Document number. Only numbers, no special
            characters.
        mtype (string): Person type. Can be either 'individual' or 'company'
        address (CreateAddressRequest): The customer's address
        metadata (dict<object, string>): Metadata
        phones (CreatePhonesRequest): TODO: type description here.
        code (string): Customer code
        gender (string): Customer Gender

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name" : "name",
        "email" : "email",
        "document" : "document",
        "mtype" : "type",
        "address" : "address",
        "metadata" : "metadata",
        "phones" : "phones",
        "code" : "code",
        "gender" : "gender"
    }

    def __init__(self,
                 name=None,
                 email=None,
                 document=None,
                 mtype=None,
                 address=None,
                 metadata=None,
                 phones=None,
                 code=None,
                 gender=None):
        """Constructor for the CreateCustomerRequest class"""

        # Initialize members of the class
        self.name = name
        self.email = email
        self.document = document
        self.mtype = mtype
        self.address = address
        self.metadata = metadata
        self.phones = phones
        self.code = code
        self.gender = gender


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name")
        email = dictionary.get("email")
        document = dictionary.get("document")
        mtype = dictionary.get("type")
        address = mundiapi.models.create_address_request.CreateAddressRequest.from_dictionary(dictionary.get("address")) if dictionary.get("address") else None
        metadata = dictionary.get("metadata")
        phones = mundiapi.models.create_phones_request.CreatePhonesRequest.from_dictionary(dictionary.get("phones")) if dictionary.get("phones") else None
        code = dictionary.get("code")
        gender = dictionary.get("gender")

        # Return an object of this model
        return cls(name,
                   email,
                   document,
                   mtype,
                   address,
                   metadata,
                   phones,
                   code,
                   gender)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_credit_card_payment_request
import mundiapi.models.create_boleto_payment_request
import mundiapi.models.create_voucher_payment_request
import mundiapi.models.create_bank_transfer_payment_request
import mundiapi.models.create_checkout_payment_request

class CreatePaymentRequest(object):

    """Implementation of the 'CreatePaymentRequest' model.

    Payment data

    Attributes:
        payment_method (string): Payment method
        credit_card (CreateCreditCardPaymentRequest): Settings for credit card
            payment
        boleto (CreateBoletoPaymentRequest): Settings for boleto payment
        currency (string): Currency. Must be informed using 3 characters
        voucher (CreateVoucherPaymentRequest): Settings for voucher payment
        metadata (dict<object, string>): Metadata
        bank_transfer (CreateBankTransferPaymentRequest): Settings for bank
            transfer payment
        gateway_affiliation_id (string): Gateway affiliation code
        amount (int): The amount of the payment, in cents
        checkout (CreateCheckoutPaymentRequest): Settings for checkout
            payment

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payment_method" : "payment_method",
        "credit_card" : "credit_card",
        "boleto" : "boleto",
        "currency" : "currency",
        "voucher" : "voucher",
        "metadata" : "metadata",
        "bank_transfer" : "bank_transfer",
        "gateway_affiliation_id" : "gateway_affiliation_id",
        "amount" : "amount",
        "checkout" : "checkout"
    }

    def __init__(self,
                 payment_method=None,
                 credit_card=None,
                 boleto=None,
                 currency=None,
                 voucher=None,
                 metadata=None,
                 bank_transfer=None,
                 gateway_affiliation_id=None,
                 amount=None,
                 checkout=None):
        """Constructor for the CreatePaymentRequest class"""

        # Initialize members of the class
        self.payment_method = payment_method
        self.credit_card = credit_card
        self.boleto = boleto
        self.currency = currency
        self.voucher = voucher
        self.metadata = metadata
        self.bank_transfer = bank_transfer
        self.gateway_affiliation_id = gateway_affiliation_id
        self.amount = amount
        self.checkout = checkout


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        payment_method = dictionary.get("payment_method")
        credit_card = mundiapi.models.create_credit_card_payment_request.CreateCreditCardPaymentRequest.from_dictionary(dictionary.get("credit_card")) if dictionary.get("credit_card") else None
        boleto = mundiapi.models.create_boleto_payment_request.CreateBoletoPaymentRequest.from_dictionary(dictionary.get("boleto")) if dictionary.get("boleto") else None
        currency = dictionary.get("currency")
        voucher = mundiapi.models.create_voucher_payment_request.CreateVoucherPaymentRequest.from_dictionary(dictionary.get("voucher")) if dictionary.get("voucher") else None
        metadata = dictionary.get("metadata")
        bank_transfer = mundiapi.models.create_bank_transfer_payment_request.CreateBankTransferPaymentRequest.from_dictionary(dictionary.get("bank_transfer")) if dictionary.get("bank_transfer") else None
        gateway_affiliation_id = dictionary.get("gateway_affiliation_id")
        amount = dictionary.get("amount")
        checkout = mundiapi.models.create_checkout_payment_request.CreateCheckoutPaymentRequest.from_dictionary(dictionary.get("checkout")) if dictionary.get("checkout") else None

        # Return an object of this model
        return cls(payment_method,
                   credit_card,
                   boleto,
                   currency,
                   voucher,
                   metadata,
                   bank_transfer,
                   gateway_affiliation_id,
                   amount,
                   checkout)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_boleto_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
from mundiapi.api_helper import APIHelper
import mundiapi.models.create_address_request

class CreateBoletoPaymentRequest(object):

    """Implementation of the 'CreateBoletoPaymentRequest' model.

    Contains the settings for creating a boleto payment

    Attributes:
        retries (int): Number of retries
        bank (string): The bank code, containing three characters. The
            available codes are on the API specification
        instructions (string): The instructions field that will be printed on
            the boleto.
        due_at (datetime): Boleto due date
        billing_address (CreateAddressRequest): Card's billing address
        billing_address_id (string): The address id for the billing address

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "retries" : "retries",
        "bank" : "bank",
        "instructions" : "instructions",
        "due_at" : "due_at",
        "billing_address" : "billing_address",
        "billing_address_id" : "billing_address_id"
    }

    def __init__(self,
                 retries=None,
                 bank=None,
                 instructions=None,
                 due_at=None,
                 billing_address=None,
                 billing_address_id=None):
        """Constructor for the CreateBoletoPaymentRequest class"""

        # Initialize members of the class
        self.retries = retries
        self.bank = bank
        self.instructions = instructions
        self.due_at = APIHelper.RFC3339DateTime(due_at) if due_at else None
        self.billing_address = billing_address
        self.billing_address_id = billing_address_id


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        retries = dictionary.get("retries")
        bank = dictionary.get("bank")
        instructions = dictionary.get("instructions")
        due_at = APIHelper.RFC3339DateTime.from_value(dictionary.get("due_at")).datetime if dictionary.get("due_at") else None
        billing_address = mundiapi.models.create_address_request.CreateAddressRequest.from_dictionary(dictionary.get("billing_address")) if dictionary.get("billing_address") else None
        billing_address_id = dictionary.get("billing_address_id")

        # Return an object of this model
        return cls(retries,
                   bank,
                   instructions,
                   due_at,
                   billing_address,
                   billing_address_id)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_phones_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_phone_request

class CreatePhonesRequest(object):

    """Implementation of the 'CreatePhonesRequest' model.

    TODO: type model description here.

    Attributes:
        home_phone (CreatePhoneRequest): TODO: type description here.
        mobile_phone (CreatePhoneRequest): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "home_phone" : "home_phone",
        "mobile_phone" : "mobile_phone"
    }

    def __init__(self,
                 home_phone=None,
                 mobile_phone=None):
        """Constructor for the CreatePhonesRequest class"""

        # Initialize members of the class
        self.home_phone = home_phone
        self.mobile_phone = mobile_phone


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        home_phone = mundiapi.models.create_phone_request.CreatePhoneRequest.from_dictionary(dictionary.get("home_phone")) if dictionary.get("home_phone") else None
        mobile_phone = mundiapi.models.create_phone_request.CreatePhoneRequest.from_dictionary(dictionary.get("mobile_phone")) if dictionary.get("mobile_phone") else None

        # Return an object of this model
        return cls(home_phone,
                   mobile_phone)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_card_token_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""


class CreateCardTokenRequest(object):

    """Implementation of the 'CreateCardTokenRequest' model.

    Card token data

    Attributes:
        number (string): Credit card number
        holder_name (string): Holder name, as written on the card
        exp_month (int): The expiration month
        exp_year (int): The expiration year, that can be informed with 2 or 4
            digits
        cvv (string): The card's security code
        brand (string): Card brand

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "number" : "number",
        "holder_name" : "holder_name",
        "exp_month" : "exp_month",
        "exp_year" : "exp_year",
        "cvv" : "cvv",
        "brand" : "brand"
    }

    def __init__(self,
                 number=None,
                 holder_name=None,
                 exp_month=None,
                 exp_year=None,
                 cvv=None,
                 brand=None):
        """Constructor for the CreateCardTokenRequest class"""

        # Initialize members of the class
        self.number = number
        self.holder_name = holder_name
        self.exp_month = exp_month
        self.exp_year = exp_year
        self.cvv = cvv
        self.brand = brand


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        number = dictionary.get("number")
        holder_name = dictionary.get("holder_name")
        exp_month = dictionary.get("exp_month")
        exp_year = dictionary.get("exp_year")
        cvv = dictionary.get("cvv")
        brand = dictionary.get("brand")

        # Return an object of this model
        return cls(number,
                   holder_name,
                   exp_month,
                   exp_year,
                   cvv,
                   brand)





# -*- coding: utf-8 -*-

"""
    mundiapi.models.create_checkout_payment_request

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io )
"""
import mundiapi.models.create_checkout_card_payment_request
import mundiapi.models.create_checkout_boleto_payment_request

class CreateCheckoutPaymentRequest(object):

    """Implementation of the 'CreateCheckoutPaymentRequest' model.

    Checkout payment request

    Attributes:
        accepted_payment_methods (list of string): Accepted Payment Methods
        success_url (string): Success url
        default_payment_method (string): Default payment method
        gateway_affiliation_id (string): Gateway Affiliation Id
        credit_card (CreateCheckoutCardPaymentRequest): Card payment request
        boleto (CreateCheckoutBoletoPaymentRequest): Boleto payment request

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "accepted_payment_methods" : "accepted_payment_methods",
        "success_url" : "success_url",
        "default_payment_method" : "default_payment_method",
        "gateway_affiliation_id" : "gateway_affiliation_id",
        "credit_card" : "credit_card",
        "boleto" : "boleto"
    }

    def __init__(self,
                 accepted_payment_methods=None,
                 success_url=None,
                 default_payment_method=None,
                 gateway_affiliation_id=None,
                 credit_card=None,
                 boleto=None):
        """Constructor for the CreateCheckoutPaymentRequest class"""

        # Initialize members of the class
        self.accepted_payment_methods = accepted_payment_methods
        self.success_url = success_url
        self.default_payment_method = default_payment_method
        self.gateway_affiliation_id = gateway_affiliation_id
        self.credit_card = credit_card
        self.boleto = boleto


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        accepted_payment_methods = dictionary.get("accepted_payment_methods")
        success_url = dictionary.get("success_url")
        default_payment_method = dictionary.get("default_payment_method")
        gateway_affiliation_id = dictionary.get("gateway_affiliation_id")
        credit_card = mundiapi.models.create_checkout_card_payment_request.CreateCheckoutCardPaymentRequest.from_dictionary(dictionary.get("credit_card")) if dictionary.get("credit_card") else None
        boleto = mundiapi.models.create_checkout_boleto_payment_request.CreateCheckoutBoletoPaymentRequest.from_dictionary(dictionary.get("boleto")) if dictionary.get("boleto") else None

        # Return an object of this model
        return cls(accepted_payment_methods,
                   success_url,
                   default_payment_method,
                   gateway_affiliation_id,
                   credit_card,
                   boleto)


